#ifndef SHARED_H
#define SHARED_H

#include "lists.h"

struct interpreter;

typedef struct shared_object_class
{
	char *id;
	void **base;
	list_destroy_t destroy;
	int size;
	int offs;
	int offs2;
} shared_object_class;

DEFLIST(exported_names,
		char *extname;
		);

DEFLIST(allow_list,
		struct interpreter *subject;
		);

typedef struct shared_object
{
		exported_names *extnames;
		int refcnt;
		allow_list *allowed;
		allow_list *denied;
		struct interpreter *owner;
		shared_object_class *class;
} shared_object;

DEFLIST(shared_int,
		char *intname;
		void *obj;
		shared_object *so;
		);

int find_byref(void *obj, void *key);
int so_find(void *obj, char *name);
int si_find_exact(shared_int *si, char *name);
int si_find_match(shared_int *si, char *key);
int si_find_byso(shared_int *si, void *so);

void destroy_shared_int(shared_int *si);
void destroy_so(shared_object *so);

void export_shared(shared_object *so, char *extname);
void *find_exported(char *clsname, char *name);
shared_int *embed_shared(void *object, char *clsname, char *name);
shared_int *import_shared(char *iname, char *name, char *clsname);

shared_int *init_shared_object(struct interpreter *i, void *so, char *clsname, char *intname);

shared_int *find_si_itself(struct interpreter *i, char *cls, char *iname);
void *quiet_find_shared_int(struct interpreter *i, char *cls, char *iname);
void *find_shared_int(struct interpreter *i, char *cls, char *iname);


void check_si_perm(struct interpreter *i, shared_int *si);
void allow_shared_int(struct interpreter *i, shared_int *si, struct interpreter *dest);
void deny_shared_int(struct interpreter *i, shared_int *si, struct interpreter *dest);

void rename_shared_int(struct interpreter *i, char *cls, char *name, char *newname, int force);

void add_shared_int(struct interpreter *i, shared_int *si);

#endif
